import { proxyCustomElement, HTMLElement, createEvent, h, Host } from '@stencil/core/internal/client';
import { Z as getBrandColorBrightness, _ as BLACK_MIN_COLOR_BRIGHTNESS, $ as WHITE_MAX_COLOR_BRIGHTNESS, j as convertToRGBA, k as getBrandForegroundAppearance } from './helpers.js';

const Theme = /*@__PURE__*/ proxyCustomElement(class Theme extends HTMLElement {
    constructor() {
        super();
        this.__registerHost();
        this.brandChange = createEvent(this, "brandChange", 7);
        this.icThemeChange = createEvent(this, "icThemeChange", 7);
        this.themeClass = "";
        /**
         * The brand colour. Can be a hex value e.g. "#ff0000", RGB e.g. "rgb(255, 0, 0)", or RGBA e.g. "rgba(255, 0, 0, 1)".
         */
        this.brandColor = null;
        /**
         * The theme mode. Can be "dark", "light", or "system". "system" will use the device or browser settings.
         */
        this.theme = "light";
        this.darkModeChangeHandler = () => {
            if (this.theme === "system") {
                this.themeClass =
                    window.matchMedia &&
                        window.matchMedia("(prefers-color-scheme: dark)").matches
                        ? "ic-theme-dark"
                        : "ic-theme-light";
            }
            else {
                this.themeClass = `ic-theme-${this.theme}`;
            }
            this.icThemeChange.emit(this.theme);
        };
        this.checkBrandColorContrast = () => {
            if (getBrandColorBrightness() < BLACK_MIN_COLOR_BRIGHTNESS &&
                getBrandColorBrightness() > WHITE_MAX_COLOR_BRIGHTNESS) {
                console.warn(`The brand colour does not provide enough contrast with either of the ICDS black or white foreground colours. Consider choosing a colour with a different brightness to achieve sufficient colour contrast for good visibility. See https://www.w3.org/TR/AERT/#color-contrast for more information about colour contrast.`);
            }
        };
        this.setBrandColor = () => {
            const colorRGBA = this.brandColor ? convertToRGBA(this.brandColor) : null;
            if (colorRGBA) {
                const { r, g, b, a } = colorRGBA;
                const { style } = document.documentElement;
                style.setProperty("--ic-brand-color-primary-r", `${r}`);
                style.setProperty("--ic-brand-color-primary-g", `${g}`);
                style.setProperty("--ic-brand-color-primary-b", `${b}`);
                style.setProperty("--ic-brand-color-primary-a", `${a}`);
                this.checkBrandColorContrast();
                this.brandChange.emit({
                    mode: getBrandForegroundAppearance(),
                    color: colorRGBA,
                });
            }
        };
    }
    watchBrandColorPropHandler() {
        this.setBrandColor();
    }
    watchThemePropHandler() {
        this.darkModeChangeHandler();
    }
    componentWillLoad() {
        this.darkModeChangeHandler();
        this.setBrandColor();
        window.matchMedia &&
            window
                .matchMedia("(prefers-color-scheme: dark)")
                .addEventListener("change", this.darkModeChangeHandler);
    }
    render() {
        const { themeClass } = this;
        return (h(Host, { key: 'd994b33e982d89fdb62837456a5a1c90471d0a6a', class: themeClass }, h("slot", { key: 'cc67f5ec282adfe4695a7ecf9134fba920a71a7e' })));
    }
    static get watchers() { return {
        "brandColor": ["watchBrandColorPropHandler"],
        "theme": ["watchThemePropHandler"]
    }; }
}, [4, "ic-theme", {
        "brandColor": [1, "brand-color"],
        "theme": [1],
        "themeClass": [32]
    }, undefined, {
        "brandColor": ["watchBrandColorPropHandler"],
        "theme": ["watchThemePropHandler"]
    }]);
function defineCustomElement$1() {
    if (typeof customElements === "undefined") {
        return;
    }
    const components = ["ic-theme"];
    components.forEach(tagName => { switch (tagName) {
        case "ic-theme":
            if (!customElements.get(tagName)) {
                customElements.define(tagName, Theme);
            }
            break;
    } });
}

const IcTheme = Theme;
const defineCustomElement = defineCustomElement$1;

export { IcTheme, defineCustomElement };

//# sourceMappingURL=ic-theme.js.map